# -*- coding: utf-8 -*-
"""
Created on Wed Jul 11 09:40:36 2018

"""
from math import *
from cmath import phase
import numpy as np
import matplotlib.pyplot as plt
from filtres import *

#==============================================================================
# Paramètre d'entrée du programme
#==============================================================================
f=1000 #Fréquence du fondamental du signal d'entrée en Hz

liste_amp=[0] + [8/(pi**2*(k)**2)*(-1)**(k//2) if k%2==1 else 0 for k in range(1,50)] #Liste des amplitudes des composantes (1ère: valeur moyenne)
liste_phase=[-pi/2 for i in range(len(liste_amp)-1)] #Liste des phases à l'orignine des composantes. (1 terme de moins que pour les amplitudes)

filtre=passehaut_1 #Nom du filtre, dans le module filtres
fc=50000 #Fréquence de coupure en Hz
#Q=5 #Facteur de qualité (le cas échéant)
A=1 #Gain dans la BP (le cas échéant)

graphe=True #Affichage du graphe (4 périodes)
spectre=True #Affichage du spectre, entrée et sortie


#==============================================================================
# Signal d'entrée décomposé en série de Fourier
#==============================================================================

def e(t):
    assert len(liste_amp)==len(liste_phase)+1
    res=liste_amp[0]
    for i in range(1,len(liste_amp)):
        res+=liste_amp[i]*cos(2*np.pi*i*f*t+liste_phase[i-1])
    return res
    
#==============================================================================
# Filtrage et création du signal de sortie par synthèse de Fourier
#==============================================================================

if filtre==passebas_1 or filtre==passehaut_1:
    def s(t,filtre):
        res=liste_amp[0]*filtre(0,fc,A)
        for i in range(1,len(liste_amp)):
            H=filtre(i*f,fc,A)
            res+=abs(H)*liste_amp[i]*cos(2*np.pi*i*f*t+liste_phase[i-1]+phase(H))
        return res

elif filtre==passebas_2 or filtre==passehaut_2 or filtre==passebande_2 or filtre==coupebande_2:
    def s(t,filtre):
        res=liste_amp[0]*filtre(0,fc,A)
        for i in range(1,len(liste_amp)):
            H=filtre(i*f,fc,Q,A)
            res+=abs(H)*liste_amp[i]*cos(2*np.pi*i*f*t+liste_phase[i-1]+phase(H))
        return res        

#==============================================================================
# Affichage du graphe         
#==============================================================================
    
if graphe:
    tabt=np.linspace(0,4/f,4000)
    tabe=[e(t) for t in tabt]
    tabs=[s(t,filtre) for t in tabt]
    plt.plot(tabt,tabe)
    plt.plot(tabt,tabs)
    plt.xlabel("$t$ en secondes")
    plt.legend(("$e(t)$","$s(t)$"))
    plt.grid()
    plt.show()
    
#==============================================================================
# Affichage du spectre en amplitude
#==============================================================================

if spectre:
    plt.subplot(121)
    for i in range(min(10,len(liste_amp))): #Seuls les 9 premiers harmoniques
        plt.plot([i*f,i*f],[0,abs(liste_amp[i])],'r')
    plt.title("Spectre de e(t)")
    plt.xlim(0,min(10,len(liste_amp))*f)
    plt.subplot(122)
    for i in range(min(10,len(liste_amp))): #Seuls les 9 premiers harmoniques
        if filtre==passebas_1 or filtre==passehaut_1: 
            H=filtre(i*f,fc,A)
            plt.plot([i*f,i*f],[0,abs(H)*abs(liste_amp[i])],'r')
        elif filtre==passebas_2 or filtre==passehaut_2 or filtre==passebande_2 or filtre==coupebande_2:
            H=filtre(i*f,fc,Q,A)
            plt.plot([i*f,i*f],[0,abs(H)*abs(liste_amp[i])],'r')
    plt.title("Spectre de s(t)")
    plt.xlim(0,min(10,len(liste_amp))*f)
    plt.show()
        